/**************************************************************************** 
	Front scattering - bake front scattering sample points as UV's 


*****************************************************************************/


//B(P,N,Pi,Ni) = 1-(1  N.Ni)(1  N.L)/2
float B(point Pp; normal Nn; point Pii; normal Nii)
{	
	vector L = Pp - Pii;
	float bounceAttenuation = 1-(1- (Nn . Nii))*(1- (Nn . L))/2;
	return bounceAttenuation;
}



surface bake_frontScatter_UVs (
		string filename = "";
		float numSamples = 50;
		float scatterRadius = 1;
		float attCoe = 0.227;
		output varying float uniqueID = 1234567;
		)
{
	
	normal Nn = normalize(N);
	normal Nf = faceforward(normalize(N), I);
	vector V = -normalize(I);
	
	color opacity = Os;
	
	string raylabel;
	rayinfo("label", raylabel);

	color Cout = 0;

	//check to see if invoked by ray we just sent
	if(raylabel == "subsurface")
	{
		Cout = diffuse(Nn);
		
	}
	else
	{
		color Csurf, Cspec;
		float backScat;
		
		vector x=normalize(dPdu)*scatterRadius;
		vector y=x^Nf*scatterRadius;
	
		color scatterCol=0;
		point Pi;
		color PiColor = 0;
		normal Ni;
		float PiU = 0;
		float PiV = 0;
		
		float numSamplesHit = 0;
		uniform float i;
		if(numSamples > 0)
		{
			for(i=0;i<numSamples;i+=1)
			{
				point PP=P+(((random()-0.5)*x)+((random()-0.5)*y));

				float objID = 0;

				gather("illuminance", PP, -Nn, 0, 1,
						"label", "subsurface",
						"surface:uniqueID", objID,
						"primitive:P", Pi,
						"primitive:N", Ni,
						"surface:u", PiU,
						"surface:v", PiV,
						"surface:Ci", PiColor)
					{	
						if(objID == uniqueID)
						{	
							//printf("%f\n", objID);


							//exclude rays that hit other side of surface
							if(N . Ni > 0)
							{

								numSamplesHit += 1;

								float bAtt = B(P, Nf, Pi, Ni);

								float dist = distance(P, Pi);
								scatterCol += PiColor * exp(-attCoe * dist); // * bAtt;
								
								
								/*********  bake   *******/
								//call dso to bake to file
								//float success = 5;
								bakeFrontScatter(u, v, PiU, PiV);
								
								
								//printf("%f\n", success);
								
							}
							
						}
					}

			}
		}
		
		if(numSamplesHit != 0)
			scatterCol = scatterCol/numSamplesHit;
			
		Cout = scatterCol;
		
		/*//bake to point cloud file
		if(filename != "")
			irradiancecache("insert", filename, "w", P, Nn, "irradiance", scatterCol);
	
		opacity = 0.5;*/
	}
	
	Oi = Os;
	Ci = Cout;
}
