
//customized fresnel function
vector efresnel(vector II; normal NN; float eta; output float Kr, Kt;)
{
	vector R, T;
	fresnel(II, NN, eta, Kr, Kt, R, T);
	Kr = smoothstep(0., .5, Kr);
	Kt = 1. - Kr;
	return normalize(T);
}

//B(P,N,Pi,Ni) = 1-(1  N.Ni)(1  N.L)/2
float B(point Pp; normal Nn; point Pii; normal Nii)
{	
	vector L = Pp - Pii;
	float bounceAttenuation = 1-(1- (Nn . Nii))*(1- (Nn . L))/2;
	return bounceAttenuation;
}

//calculate back scattering
color backScatter(point Pin; normal Nnin; float Ksub, extcoeff, samples; color absColor)
    {
       
 
	color backScatterCol=0;	
	
        color raycolor = 0, sum = 0, irrad;
        float dist = 0;
        

            /* Add subsurface scattering */
            if (Ksub > 0) {
                gather("illuminance", Pin, -Nnin, PI/4, samples,
                       "label", "subsurface",
                       "surface:Ci", raycolor, "ray:length", dist) {
                    sum += exp(-extcoeff*dist) * raycolor;
                }
                irrad = sum / samples;
                backScatterCol += Ksub * irrad * absColor;
            }
        
       return backScatterCol;
    }


surface skinModel (
		float numSamples = 50;
		float globalScale = 1;
		
		string ColorMap = "";
    		float ColorMapBlur = 1;
    		
    		float kDiffuse = .8;
		string DiffuseMap = "";
    		float DiffuseMapBlur = 1;
    		
    		float kSpecular = .3;
		string SpecularMap = "";
    		float SpecularMapBlur = 1;
    		
    		color SpecularColor = color(1,1,1);
    		float roughness = 0.3;
    		float edgeReflectance = 0.5;
    		
		float scatterRadius = 1;
		float kFrontScatter = 1;
		float mixAmount = 0.5;
		float attCoe = 0.227;
		varying color tint = (1, 0, 0);
		
		float kBackScatter = 0.6;
		string BackScatterMap = "";
		float backScatterMapBlur = 1;
		color backScatterTint = (1, 0, 0);
		float extcoeff = 0.227;
		float samples = 16;
		
		point STMatrix0 = point "shader" (1,0,0);
    		point STMatrix1 = point "shader" (0,1,0);
    		
		output varying float uniqueID = 1234567;
		)
{
	
	normal Nf = faceforward(normalize(N), I);
	vector V = -normalize(I);
	
	string raylabel;
	rayinfo("label", raylabel);

	color Cout = 0;

	//check to see if invoked by ray we just sent
	if(raylabel == "subsurface")
	{
		Cout = diffuse(Nf);
	}
	else
	{
		color Csurf, Cspec;
		float diff, spec, backScat;
	
		vector x=normalize(dPdu)*scatterRadius;
		vector y=x^Nf*scatterRadius;
	
	/* ****************************************************************************************
	*	Sort through maps assigning:
	*	surface color, diffuse factor, specular factor, specular color, 
	*	fresnel specular factor, back scatter factor
	****************************************************************************************** */
	
		/* apply STMatrix to texture coordinates*/
		float ss, tt;
		ss = vector(s, t, 1) . vector(transform("shader", STMatrix0));
    		tt = vector(s, t, 1) . vector(transform("shader", STMatrix1));
		
		/*determine surface color*/
		if(ColorMap != "")
		{
		   Csurf =  color texture(ColorMap, ss, tt,
		                            "swidth", ColorMapBlur,
		                            "twidth", ColorMapBlur );
		}
		else
		{
		 Csurf = Cs;
    		}
	

		/*determine diffuse*/
		if(DiffuseMap != "")
		{
			diff = kDiffuse * float texture(DiffuseMap, ss, tt, 
		                            "swidth", DiffuseMapBlur,
		                            "twidth", DiffuseMapBlur );
		}
		else
		{
		        diff = kDiffuse;
    		}
		
		
		/*determine specular*/
		if(SpecularMap != "")
		{
			spec = kSpecular * float texture(SpecularMap, ss, tt, 
		                            "swidth", SpecularMapBlur,
		                            "twidth", SpecularMapBlur );
		}
		else
		{
		        spec = kSpecular;
    		}
		
		/*determine specular color*/
		Cspec = SpecularColor;
		
		/*determine fresnel specluar*/
		float Kr, Kt;
		vector T = efresnel(-V, Nf, 0.7, Kr, Kt);
		Kr *= edgeReflectance;
		
		color edgeReflectanceFactor = 0;
		illuminance(P, Nf, PI/2)
			{
				vector Ln = normalize(L);
				
				//add glossy/diffuse reflection at edges
				vector H = normalize(Ln + V);
				
				if (H . Nf > 0)
					edgeReflectanceFactor += Kr  * Cl * (Ln . Nf) * pow(H . Nf, 4.);	
			}
		
		/*determine back scatter factor*/
		if(BackScatterMap != "")
		{
			backScat = kBackScatter * float texture(BackScatterMap, ss, tt,
								"swidth", backScatterMapBlur,
								"twidth", backScatterMapBlur );
		}
		else
		{
			backScat = kBackScatter;
		}
								
	/* ****************************************************************************************
		calculate front scattering
	******************************************************************************************* */
		
		
		color scatterCol=0;
		point Pi;
		color PiColor = 0;
		normal Ni;
		
		float numSamplesHit = 0;
		uniform float i;
		if((numSamples > 0) || (kFrontScatter > 0))
		{
			for(i=0;i<numSamples;i+=1)
			{
				point PP=P+(((random()-0.5)*x)+((random()-0.5)*y));

				float objID = 0;

				gather("illuminance", PP, -Nf, 0, 1,
						"label", "subsurface",
						"surface:uniqueID", objID,
						"primitive:P", Pi,
						"primitive:N", Ni,
						"surface:Ci", PiColor)
					{	
						if(objID == uniqueID)
						{	
							//printf("%f\n", objID);


							//exclude rays that hit other side of surface
							if(N . Ni > 0)
							{

								numSamplesHit += 1;

								float bAtt = B(P, Nf, Pi, Ni);

								float dist = distance(P, Pi);
								scatterCol += PiColor * exp(-attCoe * dist); // * bAtt;

							}
							
						}
					}

			}
		}
		
		if(numSamplesHit != 0)
			scatterCol = scatterCol/numSamplesHit;
		
		
		
	/***********************************************************************************************************/
	
		color backScatterColor = 0;
		if(backScat != 0)
			backScatterColor = backScat * (backScatter(P, N, 1, extcoeff, samples, backScatterTint));
			
		Cout = globalScale * 
				((ambient() + Csurf * diff * mix((diffuse(Nf)),(scatterCol * tint * kFrontScatter + backScatterColor), mixAmount))
				+ (spec * Cspec * (specular(Nf, V, roughness)) + edgeReflectanceFactor));
					
	}
	
	Oi = Os;
	Ci = Cout;
}