
//customized fresnel function
vector efresnel(vector II; normal NN; float eta; output float Kr, Kt;)
{
	vector R, T;
	fresnel(II, NN, eta, Kr, Kt, R, T);
	Kr = smoothstep(0., .5, Kr);
	Kt = 1. - Kr;
	return normalize(T);
}


surface skinModel_usingBakedData (
		float globalScale = 1;
		
		string ColorMap = "";
    		float ColorMapBlur = 1;
    		
    		float kDiffuse = .8;
		string DiffuseMap = "";
    		float DiffuseMapBlur = 1;
    		
    		float kSpecular = .3;
		string SpecularMap = "";
    		float SpecularMapBlur = 1;
    		
    		color SpecularColor = color(1,1,1);
    		float roughness = 0.3;
    		float edgeReflectance = 0.5;
    		
    		string bakedFrontScatterMap = "";
    		color frontScatterTint = (1, 0.73, 0.73);
		float kFrontScatter = 1;
		float mixAmount = 0.5;
		
		string bakedBackScatterMap = "";
		float kBackScatter = 0.6;
		string BackScatterMap = "";
		float backScatterMapBlur = 1;
		color backScatterTint = (1, 0, 0);
		
		
		point STMatrix0 = point "shader" (1,0,0);
    		point STMatrix1 = point "shader" (0,1,0);
    		
		)
{
	
	normal Nf = faceforward(normalize(N), I);
	vector V = -normalize(I);
	
	color Cout = 0;

	
	
		color Csurf, Cspec;
		float diff, spec, backScat;
	
	/* ****************************************************************************************
	*	Sort through maps assigning:
	*	surface color, diffuse factor, specular factor, specular color, 
	*	fresnel specular factor, back scatter factor
	****************************************************************************************** */
	
		/* apply STMatrix to texture coordinates*/
		float ss, tt;
		ss = vector(s, t, 1) . vector(transform("shader", STMatrix0));
    		tt = vector(s, t, 1) . vector(transform("shader", STMatrix1));
		
		/*determine surface color*/
		if(ColorMap != "")
		{
		   Csurf =  color texture(ColorMap, ss, tt,
		                            "swidth", ColorMapBlur,
		                            "twidth", ColorMapBlur );
		}
		else
		{
		 Csurf = Cs;
    		}
	

		/*determine diffuse*/
		if(DiffuseMap != "")
		{
			diff = kDiffuse * float texture(DiffuseMap, ss, tt, 
		                            "swidth", DiffuseMapBlur,
		                            "twidth", DiffuseMapBlur );
		}
		else
		{
		        diff = kDiffuse;
    		}
		
		
		/*determine specular*/
		if(SpecularMap != "")
		{
			spec = kSpecular * float texture(SpecularMap, ss, tt, 
		                            "swidth", SpecularMapBlur,
		                            "twidth", SpecularMapBlur );
		}
		else
		{
		        spec = kSpecular;
    		}
		
		/*determine specular color*/
		Cspec = SpecularColor;
		
		/*determine fresnel specluar*/
		float Kr, Kt;
		vector T = efresnel(-V, Nf, 0.7, Kr, Kt);
		Kr *= edgeReflectance;
		
		color edgeReflectanceFactor = 0;
		illuminance(P, Nf, PI/2)
			{
				vector Ln = normalize(L);
				
				//add glossy/diffuse reflection at edges
				vector H = normalize(Ln + V);
				
				if (H . Nf > 0)
					edgeReflectanceFactor += Kr  * Cl * (Ln . Nf) * pow(H . Nf, 4.);	
			}
		
		/*determine back scatter factor*/
		if(BackScatterMap != "")
		{
			backScat = kBackScatter * float texture(BackScatterMap, ss, tt,
								"swidth", backScatterMapBlur,
								"twidth", backScatterMapBlur );
		}
		else
		{
			backScat = kBackScatter;
		}
								
		/**************************************************************************************/
		
		
		//TO DO... 
		//pass in uv to dso,
		//first time called, builds list/queue of scatterUV's, and returns the num of uvs
		//loop and call again, returning sample uv
			//within loop, call another dso to lookup up P for uv, returns Pi
			//calculate diffuse illuminance at Pi
		
		
		//lookup precomputed front scatter contribution	
		color scatterCol = 0;
		float maxdist = 1;
		if(bakedFrontScatterMap != "")
			irradiancecache("query", bakedFrontScatterMap, "r", P, N, "maxdist", maxdist, "irradiance", scatterCol);

		//lookup precomputed back scatter contribution
		color backScattCol = 0;
		if(bakedBackScatterMap != "")
			irradiancecache("query", bakedBackScatterMap, "r", P, N, "maxdist", maxdist, "irradiance", backScattCol);

	
		Cout = globalScale * 
				((ambient() + Csurf * diff * mix((diffuse(Nf)),(scatterCol * frontScatterTint * kFrontScatter + backScat * backScattCol * backScatterTint), mixAmount))
				+ (spec * Cspec * (specular(Nf, V, roughness)) + edgeReflectanceFactor));
	
	Oi = Os;
	Ci = Cout;
}